@@ -1,29 +1,40 @@
 """
 Support for installing and building the "wheel" binary package format.
 """
-from __future__ import with_statement
+from __future__ import absolute_import
 
 import compileall
 import csv
+import errno
 import functools
 import hashlib
+import logging
 import os
+import os.path
 import re
 import shutil
+import stat
 import sys
+import tempfile
+import warnings
 
 from base64 import urlsafe_b64encode
 from email.parser import Parser
 
-from pip.backwardcompat import ConfigParser, StringIO
+from pip._vendor.six import StringIO
+
+import pip
+from pip.download import path_to_url, unpack_url
 from pip.exceptions import InvalidWheelFilename, UnsupportedWheel
-from pip.locations import distutils_scheme
-from pip.log import logger
+from pip.locations import distutils_scheme, PIP_DELETE_MARKER_FILENAME
 from pip import pep425tags
-from pip.util import call_subprocess, normalize_path, make_path_relative
-from pip._vendor import pkg_resources
+from pip.utils import (
+    call_subprocess, ensure_dir, make_path_relative, captured_stdout,
+    rmtree)
+from pip.utils.logging import indent_log
 from pip._vendor.distlib.scripts import ScriptMaker
 from pip._vendor import pkg_resources
+from pip._vendor.six.moves import configparser
 
 
 wheel_ext = '.whl'
@@ -31,7 +42,108 @@ wheel_ext = '.whl'
 VERSION_COMPATIBLE = (1, 0)
 
 
-def rehash(path, algo='sha256', blocksize=1<<20):
+logger = logging.getLogger(__name__)
+
+
+class WheelCache(object):
+    """A cache of wheels for future installs."""
+
+    def __init__(self, cache_dir, format_control):
+        """Create a wheel cache.
+
+        :param cache_dir: The root of the cache.
+        :param format_control: A pip.index.FormatControl object to limit
+            binaries being read from the cache.
+        """
+        self._cache_dir = os.path.expanduser(cache_dir) if cache_dir else None
+        self._format_control = format_control
+
+    def cached_wheel(self, link, package_name):
+        return cached_wheel(
+            self._cache_dir, link, self._format_control, package_name)
+
+
+def _cache_for_link(cache_dir, link):
+    """
+    Return a directory to store cached wheels in for link.
+
+    Because there are M wheels for any one sdist, we provide a directory
+    to cache them in, and then consult that directory when looking up
+    cache hits.
+
+    We only insert things into the cache if they have plausible version
+    numbers, so that we don't contaminate the cache with things that were not
+    unique. E.g. ./package might have dozens of installs done for it and build
+    a version of 0.0...and if we built and cached a wheel, we'd end up using
+    the same wheel even if the source has been edited.
+
+    :param cache_dir: The cache_dir being used by pip.
+    :param link: The link of the sdist for which this will cache wheels.
+    """
+
+    # We want to generate an url to use as our cache key, we don't want to just
+    # re-use the URL because it might have other items in the fragment and we
+    # don't care about those.
+    key_parts = [link.url_without_fragment]
+    if link.hash_name is not None and link.hash is not None:
+        key_parts.append("=".join([link.hash_name, link.hash]))
+    key_url = "#".join(key_parts)
+
+    # Encode our key url with sha224, we'll use this because it has similar
+    # security properties to sha256, but with a shorter total output (and thus
+    # less secure). However the differences don't make a lot of difference for
+    # our use case here.
+    hashed = hashlib.sha224(key_url.encode()).hexdigest()
+
+    # We want to nest the directories some to prevent having a ton of top level
+    # directories where we might run out of sub directories on some FS.
+    parts = [hashed[:2], hashed[2:4], hashed[4:6], hashed[6:]]
+
+    # Inside of the base location for cached wheels, expand our parts and join
+    # them all together.
+    return os.path.join(cache_dir, "wheels", *parts)
+
+
+def cached_wheel(cache_dir, link, format_control, package_name):
+    if not cache_dir:
+        return link
+    if not link:
+        return link
+    if link.is_wheel:
+        return link
+    if not link.is_artifact:
+        return link
+    if not package_name:
+        return link
+    canonical_name = pkg_resources.safe_name(package_name).lower()
+    formats = pip.index.fmt_ctl_formats(format_control, canonical_name)
+    if "binary" not in formats:
+        return link
+    root = _cache_for_link(cache_dir, link)
+    try:
+        wheel_names = os.listdir(root)
+    except OSError as e:
+        if e.errno == errno.ENOENT:
+            return link
+        raise
+    candidates = []
+    for wheel_name in wheel_names:
+        try:
+            wheel = Wheel(wheel_name)
+        except InvalidWheelFilename:
+            continue
+        if not wheel.supported():
+            # Built for a different python/arch/etc
+            continue
+        candidates.append((wheel.support_index_min(), wheel_name))
+    if not candidates:
+        return link
+    candidates.sort()
+    path = os.path.join(root, candidates[0][1])
+    return pip.index.Link(path_to_url(path), trusted=True)
+
+
+def rehash(path, algo='sha256', blocksize=1 << 20):
     """Return (hash, length) for path using hashlib.new(algo)"""
     h = hashlib.new(algo)
     length = 0
@@ -41,55 +153,43 @@ def rehash(path, algo='sha256', blocksize=1<<20):
             length += len(block)
             h.update(block)
             block = f.read(blocksize)
-    digest = 'sha256='+urlsafe_b64encode(h.digest()).decode('latin1').rstrip('=')
+    digest = 'sha256=' + urlsafe_b64encode(
+        h.digest()
+    ).decode('latin1').rstrip('=')
     return (digest, length)
 
-try:
-    unicode
-    def binary(s):
-        if isinstance(s, unicode):
-            return s.encode('ascii')
-        return s
-except NameError:
-    def binary(s):
-        if isinstance(s, str):
-            return s.encode('ascii')
 
 def open_for_csv(name, mode):
     if sys.version_info[0] < 3:
         nl = {}
         bin = 'b'
     else:
-        nl = { 'newline': '' }
+        nl = {'newline': ''}
         bin = ''
     return open(name, mode + bin, **nl)
 
+
 def fix_script(path):
     """Replace #!python with #!/path/to/python
     Return True if file was changed."""
     # XXX RECORD hashes will need to be updated
     if os.path.isfile(path):
-        script = open(path, 'rb')
-        try:
+        with open(path, 'rb') as script:
             firstline = script.readline()
-            if not firstline.startswith(binary('#!python')):
+            if not firstline.startswith(b'#!python'):
                 return False
             exename = sys.executable.encode(sys.getfilesystemencoding())
-            firstline = binary('#!') + exename + binary(os.linesep)
+            firstline = b'#!' + exename + os.linesep.encode("ascii")
             rest = script.read()
-        finally:
-            script.close()
-        script = open(path, 'wb')
-        try:
+        with open(path, 'wb') as script:
             script.write(firstline)
             script.write(rest)
-        finally:
-            script.close()
         return True
 
 dist_info_re = re.compile(r"""^(?P<namever>(?P<name>.+?)(-(?P<ver>\d.+?))?)
                                 \.dist-info$""", re.VERBOSE)
 
+
 def root_is_purelib(name, wheeldir):
     """
     Return True if the extracted wheel in wheeldir should go into purelib.
@@ -121,7 +221,7 @@ def get_entrypoints(filename):
             data.write("\n")
         data.seek(0)
 
-    cp = ConfigParser.RawConfigParser()
+    cp = configparser.RawConfigParser()
     cp.readfp(data)
 
     console = {}
@@ -134,11 +234,13 @@ def get_entrypoints(filename):
 
 
 def move_wheel_files(name, req, wheeldir, user=False, home=None, root=None,
-                     pycompile=True, scheme=None):
+                     pycompile=True, scheme=None, isolated=False):
     """Install a wheel"""
 
     if not scheme:
-        scheme = distutils_scheme(name, user=user, home=home, root=root)
+        scheme = distutils_scheme(
+            name, user=user, home=home, root=root, isolated=isolated
+        )
 
     if root_is_purelib(name, wheeldir):
         lib_dir = scheme['purelib']
@@ -159,7 +261,11 @@ def move_wheel_files(name, req, wheeldir, user=False, home=None, root=None,
 
     # Compile all of the pyc files that we're going to be installing
     if pycompile:
-        compileall.compile_dir(source, force=True, quiet=True)
+        with captured_stdout() as stdout:
+            with warnings.catch_warnings():
+                warnings.filterwarnings('ignore')
+                compileall.compile_dir(source, force=True, quiet=True)
+        logger.debug(stdout.getvalue())
 
     def normpath(src, p):
         return make_path_relative(src, p).replace(os.path.sep, '/')
@@ -173,8 +279,7 @@ def move_wheel_files(name, req, wheeldir, user=False, home=None, root=None,
             changed.add(destfile)
 
     def clobber(source, dest, is_base, fixer=None, filter=None):
-        if not os.path.exists(dest): # common for the 'include' path
-            os.makedirs(dest)
+        ensure_dir(dest)  # common for the 'include' path
 
         for dir, subdirs, files in os.walk(source):
             basedir = dir[len(source):].lstrip(os.path.sep)
@@ -186,10 +291,11 @@ def move_wheel_files(name, req, wheeldir, user=False, home=None, root=None,
                 if is_base and basedir == '' and destsubdir.endswith('.data'):
                     data_dirs.append(s)
                     continue
-                elif (is_base
-                    and s.endswith('.dist-info')
-                    # is self.req.project_name case preserving?
-                    and s.lower().startswith(req.project_name.replace('-', '_').lower())):
+                elif (is_base and
+                        s.endswith('.dist-info') and
+                        # is self.req.project_name case preserving?
+                        s.lower().startswith(
+                            req.project_name.replace('-', '_').lower())):
                     assert not info_dir, 'Multiple .dist-info directories'
                     info_dir.append(destsubdir)
             for f in files:
@@ -201,12 +307,30 @@ def move_wheel_files(name, req, wheeldir, user=False, home=None, root=None,
                 # directory creation is lazy and after the file filtering above
                 # to ensure we don't install empty dirs; empty dirs can't be
                 # uninstalled.
-                if not os.path.exists(destdir):
-                    os.makedirs(destdir)
-                # use copy2 (not move) to be extra sure we're not moving
-                # directories over; copy2 fails for directories.  this would
-                # fail tests (not during released/user execution)
-                shutil.copy2(srcfile, destfile)
+                ensure_dir(destdir)
+
+                # We use copyfile (not move, copy, or copy2) to be extra sure
+                # that we are not moving directories over (copyfile fails for
+                # directories) as well as to ensure that we are not copying
+                # over any metadata because we want more control over what
+                # metadata we actually copy over.
+                shutil.copyfile(srcfile, destfile)
+
+                # Copy over the metadata for the file, currently this only
+                # includes the atime and mtime.
+                st = os.stat(srcfile)
+                if hasattr(os, "utime"):
+                    os.utime(destfile, (st.st_atime, st.st_mtime))
+
+                # If our file is executable, then make our destination file
+                # executable.
+                if os.access(srcfile, os.X_OK):
+                    st = os.stat(srcfile)
+                    permissions = (
+                        st.st_mode | stat.S_IXUSR | stat.S_IXGRP | stat.S_IXOTH
+                    )
+                    os.chmod(destfile, permissions)
+
                 changed = False
                 if fixer:
                     changed = fixer(destfile)
@@ -248,6 +372,10 @@ def move_wheel_files(name, req, wheeldir, user=False, home=None, root=None,
 
     maker = ScriptMaker(None, scheme['scripts'])
 
+    # Ensure old scripts are overwritten.
+    # See https://github.com/pypa/pip/issues/1800
+    maker.clobber = True
+
     # Ensure we don't generate any variants for scripts because this is almost
     # never what somebody wants.
     # See https://bitbucket.org/pypa/distlib/issue/35/
@@ -339,16 +467,24 @@ if __name__ == '__main__':
         spec = 'easy_install-%s = %s' % (sys.version[:3], easy_install_script)
         generated.extend(maker.make(spec))
         # Delete any other versioned easy_install entry points
-        easy_install_ep = [k for k in console
-                if re.match(r'easy_install(-\d\.\d)?$', k)]
+        easy_install_ep = [
+            k for k in console if re.match(r'easy_install(-\d\.\d)?$', k)
+        ]
         for k in easy_install_ep:
             del console[k]
 
     # Generate the console and GUI entry points specified in the wheel
     if len(console) > 0:
-        generated.extend(maker.make_multiple(['%s = %s' % kv for kv in console.items()]))
+        generated.extend(
+            maker.make_multiple(['%s = %s' % kv for kv in console.items()])
+        )
     if len(gui) > 0:
-        generated.extend(maker.make_multiple(['%s = %s' % kv for kv in gui.items()], {'gui': True}))
+        generated.extend(
+            maker.make_multiple(
+                ['%s = %s' % kv for kv in gui.items()],
+                {'gui': True}
+            )
+        )
 
     record = os.path.join(info_dir[0], 'RECORD')
     temp_record = os.path.join(info_dir[0], 'RECORD.pip')
@@ -368,6 +504,7 @@ if __name__ == '__main__':
                 writer.writerow((installed[f], '', ''))
     shutil.move(temp_record, record)
 
+
 def _unique(fn):
     @functools.wraps(fn)
     def unique(*args, **kw):
@@ -378,6 +515,7 @@ def _unique(fn):
                 yield item
     return unique
 
+
 # TODO: this goes somewhere besides the wheel module
 @_unique
 def uninstallation_paths(dist):
@@ -389,7 +527,7 @@ def uninstallation_paths(dist):
 
     UninstallPathSet.add() takes care of the __pycache__ .pyc.
     """
-    from pip.req import FakeFile # circular import
+    from pip.utils import FakeFile  # circular import
     r = csv.reader(FakeFile(dist.get_metadata_lines('RECORD')))
     for row in r:
         path = os.path.join(dist.location, row[0])
@@ -397,7 +535,7 @@ def uninstallation_paths(dist):
         if path.endswith('.py'):
             dn, fn = os.path.split(path)
             base = fn[:-3]
-            path = os.path.join(dn, base+'.pyc')
+            path = os.path.join(dn, base + '.pyc')
             yield path
 
 
@@ -443,8 +581,10 @@ def check_compatibility(version, name):
             "of pip" % (name, '.'.join(map(str, version)))
         )
     elif version > VERSION_COMPATIBLE:
-        logger.warn('Installing from a newer Wheel-Version (%s)'
-                    % '.'.join(map(str, version)))
+        logger.warning(
+            'Installing from a newer Wheel-Version (%s)',
+            '.'.join(map(str, version)),
+        )
 
 
 class Wheel(object):
@@ -453,10 +593,11 @@ class Wheel(object):
     # TODO: maybe move the install code into this class
 
     wheel_file_re = re.compile(
-                r"""^(?P<namever>(?P<name>.+?)-(?P<ver>\d.*?))
-                ((-(?P<build>\d.*?))?-(?P<pyver>.+?)-(?P<abi>.+?)-(?P<plat>.+?)
-                \.whl|\.dist-info)$""",
-                re.VERBOSE)
+        r"""^(?P<namever>(?P<name>.+?)-(?P<ver>\d.*?))
+        ((-(?P<build>\d.*?))?-(?P<pyver>.+?)-(?P<abi>.+?)-(?P<plat>.+?)
+        \.whl|\.dist-info)$""",
+        re.VERBOSE
+    )
 
     def __init__(self, filename):
         """
@@ -464,7 +605,9 @@ class Wheel(object):
         """
         wheel_info = self.wheel_file_re.match(filename)
         if not wheel_info:
-            raise InvalidWheelFilename("%s is not a valid wheel filename." % filename)
+            raise InvalidWheelFilename(
+                "%s is not a valid wheel filename." % filename
+            )
         self.filename = filename
         self.name = wheel_info.group('name').replace('_', '-')
         # we'll assume "_" means "-" due to wheel naming scheme
@@ -475,8 +618,10 @@ class Wheel(object):
         self.plats = wheel_info.group('plat').split('.')
 
         # All the tag combinations from this file
-        self.file_tags = set((x, y, z) for x in self.pyversions for y
-                            in self.abis for z in self.plats)
+        self.file_tags = set(
+            (x, y, z) for x in self.pyversions
+            for y in self.abis for z in self.plats
+        )
 
     def support_index_min(self, tags=None):
         """
@@ -485,14 +630,14 @@ class Wheel(object):
         and one of the file tags is first in the list, then return 0.  Returns
         None is the wheel is not supported.
         """
-        if tags is None: # for mock
+        if tags is None:  # for mock
             tags = pep425tags.supported_tags
         indexes = [tags.index(c) for c in self.file_tags if c in tags]
         return min(indexes) if indexes else None
 
     def supported(self, tags=None):
         """Is this wheel supported on this system?"""
-        if tags is None: # for mock
+        if tags is None:  # for mock
             tags = pep425tags.supported_tags
         return bool(set(tags).intersection(self.file_tags))
 
@@ -500,61 +645,155 @@ class Wheel(object):
 class WheelBuilder(object):
     """Build wheels from a RequirementSet."""
 
-    def __init__(self, requirement_set, finder, wheel_dir, build_options=[], global_options=[]):
+    def __init__(self, requirement_set, finder, build_options=None,
+                 global_options=None):
         self.requirement_set = requirement_set
         self.finder = finder
-        self.wheel_dir = normalize_path(wheel_dir)
-        self.build_options = build_options
-        self.global_options = global_options
+        self._cache_root = requirement_set._wheel_cache._cache_dir
+        self._wheel_dir = requirement_set.wheel_download_dir
+        self.build_options = build_options or []
+        self.global_options = global_options or []
 
-    def _build_one(self, req):
-        """Build one wheel."""
+    def _build_one(self, req, output_dir):
+        """Build one wheel.
 
+        :return: The filename of the built wheel, or None if the build failed.
+        """
+        tempd = tempfile.mkdtemp('pip-wheel-')
+        try:
+            if self.__build_one(req, tempd):
+                try:
+                    wheel_name = os.listdir(tempd)[0]
+                    wheel_path = os.path.join(output_dir, wheel_name)
+                    shutil.move(os.path.join(tempd, wheel_name), wheel_path)
+                    logger.info('Stored in directory: %s', output_dir)
+                    return wheel_path
+                except:
+                    return None
+            return None
+        finally:
+            rmtree(tempd)
+
+    def __build_one(self, req, tempd):
         base_args = [
             sys.executable, '-c',
-            "import setuptools;__file__=%r;"\
-            "exec(compile(open(__file__).read().replace('\\r\\n', '\\n'), __file__, 'exec'))" % req.setup_py] + \
-            list(self.global_options)
+            "import setuptools;__file__=%r;"
+            "exec(compile(open(__file__).read().replace('\\r\\n', '\\n'), "
+            "__file__, 'exec'))" % req.setup_py
+        ] + list(self.global_options)
 
-        logger.notify('Running setup.py bdist_wheel for %s' % req.name)
-        logger.notify('Destination directory: %s' % self.wheel_dir)
-        wheel_args = base_args + ['bdist_wheel', '-d', self.wheel_dir] + self.build_options
+        logger.info('Running setup.py bdist_wheel for %s', req.name)
+        logger.debug('Destination directory: %s', tempd)
+        wheel_args = base_args + ['bdist_wheel', '-d', tempd] \
+            + self.build_options
         try:
             call_subprocess(wheel_args, cwd=req.source_dir, show_stdout=False)
             return True
         except:
-            logger.error('Failed building wheel for %s' % req.name)
+            logger.error('Failed building wheel for %s', req.name)
             return False
 
-    def build(self):
-        """Build wheels."""
+    def build(self, autobuilding=False):
+        """Build wheels.
 
-        #unpack and constructs req set
+        :param unpack: If True, replace the sdist we built from the with the
+            newly built wheel, in preparation for installation.
+        :return: True if all the wheels built correctly.
+        """
+        assert self._wheel_dir or (autobuilding and self._cache_root)
+        # unpack sdists and constructs req set
         self.requirement_set.prepare_files(self.finder)
 
         reqset = self.requirement_set.requirements.values()
 
-        buildset = [req for req in reqset if not req.is_wheel]
+        buildset = []
+        for req in reqset:
+            if req.is_wheel:
+                if not autobuilding:
+                    logger.info(
+                        'Skipping %s, due to already being wheel.', req.name)
+            elif req.editable:
+                if not autobuilding:
+                    logger.info(
+                        'Skipping bdist_wheel for %s, due to being editable',
+                        req.name)
+            elif autobuilding and req.link and not req.link.is_artifact:
+                pass
+            elif autobuilding and not req.source_dir:
+                pass
+            else:
+                if autobuilding:
+                    link = req.link
+                    base, ext = link.splitext()
+                    if pip.index.egg_info_matches(base, None, link) is None:
+                        # Doesn't look like a package - don't autobuild a wheel
+                        # because we'll have no way to lookup the result sanely
+                        continue
+                    if "binary" not in pip.index.fmt_ctl_formats(
+                            self.finder.format_control,
+                            pkg_resources.safe_name(req.name).lower()):
+                        logger.info(
+                            "Skipping bdist_wheel for %s, due to binaries "
+                            "being disabled for it.", req.name)
+                        continue
+                buildset.append(req)
 
         if not buildset:
-            return
+            return True
 
-        #build the wheels
-        logger.notify(
-            'Building wheels for collected packages: %s' %
-            ','.join([req.name for req in buildset])
+        # Build the wheels.
+        logger.info(
+            'Building wheels for collected packages: %s',
+            ', '.join([req.name for req in buildset]),
         )
-        logger.indent += 2
-        build_success, build_failure = [], []
-        for req in buildset:
-            if self._build_one(req):
-                build_success.append(req)
-            else:
-                build_failure.append(req)
-        logger.indent -= 2
+        with indent_log():
+            build_success, build_failure = [], []
+            for req in buildset:
+                if autobuilding:
+                    output_dir = _cache_for_link(self._cache_root, req.link)
+                    ensure_dir(output_dir)
+                else:
+                    output_dir = self._wheel_dir
+                wheel_file = self._build_one(req, output_dir)
+                if wheel_file:
+                    build_success.append(req)
+                    if autobuilding:
+                        # XXX: This is mildly duplicative with prepare_files,
+                        # but not close enough to pull out to a single common
+                        # method.
+                        # The code below assumes temporary source dirs -
+                        # prevent it doing bad things.
+                        if req.source_dir and not os.path.exists(os.path.join(
+                                req.source_dir, PIP_DELETE_MARKER_FILENAME)):
+                            raise AssertionError(
+                                "bad source dir - missing marker")
+                        # Delete the source we built the wheel from
+                        req.remove_temporary_source()
+                        # set the build directory again - name is known from
+                        # the work prepare_files did.
+                        req.source_dir = req.build_location(
+                            self.requirement_set.build_dir)
+                        # Update the link for this.
+                        req.link = pip.index.Link(
+                            path_to_url(wheel_file), trusted=True)
+                        assert req.link.is_wheel
+                        # extract the wheel into the dir
+                        unpack_url(
+                            req.link, req.source_dir, None, False,
+                            session=self.requirement_set.session)
+                else:
+                    build_failure.append(req)
 
-        #notify sucess/failure
+        # notify success/failure
         if build_success:
-            logger.notify('Successfully built %s' % ' '.join([req.name for req in build_success]))
+            logger.info(
+                'Successfully built %s',
+                ' '.join([req.name for req in build_success]),
+            )
         if build_failure:
-            logger.notify('Failed to build %s' % ' '.join([req.name for req in build_failure]))
+            logger.info(
+                'Failed to build %s',
+                ' '.join([req.name for req in build_failure]),
+            )
+        # Return True if all builds were successful
+        return len(build_failure) == 0
